@@ -23,7 +23,7 @@
 #include <errno.h>
 #include <string.h>
 #include <dirent.h>
-#include <sys/fcntl.h>
+#include <fcntl.h>
 
 #include "libbridge.h"
 #include "libbridge_private.h"
@@ -40,11 +40,15 @@ static void fetch_id(const char *dev, const char *name, struct bridge_id *id)
 {
 	FILE *f = fpopen(dev, name);
 
-	fscanf(f, "%2hhx%2hhx.%2hhx%2hhx%2hhx%2hhx%2hhx%2hhx",
-	      &id->prio[0], &id->prio[1],
-	      &id->addr[0], &id->addr[1], &id->addr[2],
-	      &id->addr[3], &id->addr[4], &id->addr[5]);
-	fclose(f);
+	if (!f)
+		fprintf(stderr, "%s: %s\n", dev, strerror(errno));
+	else {
+		fscanf(f, "%2hhx%2hhx.%2hhx%2hhx%2hhx%2hhx%2hhx%2hhx",
+		       &id->prio[0], &id->prio[1],
+		       &id->addr[0], &id->addr[1], &id->addr[2],
+		       &id->addr[3], &id->addr[4], &id->addr[5]);
+		fclose(f);
+	}
 }
 
 /* Fetch an integer attribute out of sysfs. */
@@ -54,11 +58,10 @@ static int fetch_int(const char *dev, const char *name)
 	int value = -1;
 
 	if (!f) 
-		fprintf(stderr, "%s: %s\n", dev, strerror(errno));
-	else {
-		fscanf(f, "%i", &value);
-		fclose(f);
-	}
+		return 0;
+
+	fscanf(f, "%i", &value);
+	fclose(f);
 	return value;
 }
 
@@ -235,6 +238,7 @@ static int old_get_port_info(const char *brname, const char *port,
 	__jiffies_to_tv(&info->forward_delay_timer_value, 
 			i.forward_delay_timer_value);
 	__jiffies_to_tv(&info->hold_timer_value, i.hold_timer_value);
+	info->hairpin_mode = 0;
 	return 0;
 }
 
@@ -267,6 +271,8 @@ int br_get_port_info(const char *brname, const char *port,
 	fetch_tv(path, "message_age_timer", &info->message_age_timer_value);
 	fetch_tv(path, "forward_delay_timer", &info->forward_delay_timer_value);
 	fetch_tv(path, "hold_timer", &info->hold_timer_value);
+	info->hairpin_mode = fetch_int(path, "hairpin_mode");
+
 	closedir(d);
 
 	return 0;
@@ -274,25 +280,38 @@ fallback:
 	return old_get_port_info(brname, port, info);
 }
 
+static int set_sysfs(const char *path, unsigned long value)
+{
+	int fd, ret = 0, cc;
+	char buf[32];
+
+	fd = open(path, O_WRONLY);
+	if (fd < 0)
+		return -1;
+
+	cc = snprintf(buf, sizeof(buf), "%lu\n", value);
+	if (write(fd, buf, cc) < 0)
+		ret = -1;
+	close(fd);
+
+	return ret;
+}
+
 
 static int br_set(const char *bridge, const char *name,
 		  unsigned long value, unsigned long oldcode)
 {
 	int ret;
 	char path[SYSFS_PATH_MAX];
-	FILE *f;
 
-	snprintf(path, SYSFS_PATH_MAX, SYSFS_CLASS_NET "%s/%s", bridge, name);
+	snprintf(path, SYSFS_PATH_MAX, SYSFS_CLASS_NET "%s/bridge/%s",
+		 bridge, name);
 
-	f = fopen(path, "w");
-	if (f) {
-		ret = fprintf(f, "%ld\n", value);
-		fclose(f);
-	} else {
+	if ((ret = set_sysfs(path, value)) < 0) {
 		/* fallback to old ioctl */
 		struct ifreq ifr;
 		unsigned long args[4] = { oldcode, value, 0, 0 };
-		
+
 		strncpy(ifr.ifr_name, bridge, IFNAMSIZ);
 		ifr.ifr_data = (char *) &args;
 		ret = ioctl(br_socket_fd, SIOCDEVPRIVATE, &ifr);
@@ -342,14 +361,10 @@ static int port_set(const char *bridge, const char *ifname,
 {
 	int ret;
 	char path[SYSFS_PATH_MAX];
-	FILE *f;
 
 	snprintf(path, SYSFS_PATH_MAX, SYSFS_CLASS_NET "%s/brport/%s", ifname, name);
-	f = fopen(path, "w");
-	if (f) {
-		ret = fprintf(f, "%ld\n", value);
-		fclose(f);
-	} else {
+
+	if ((ret = set_sysfs(path, value)) < 0) {
 		int index = get_portno(bridge, ifname);
 
 		if (index < 0)
@@ -377,6 +392,11 @@ int br_set_path_cost(const char *bridge, const char *port, int cost)
 	return port_set(bridge, port, "path_cost", cost, BRCTL_SET_PATH_COST);
 }
 
+int br_set_hairpin_mode(const char *bridge, const char *port, int hairpin_mode)
+{
+	return port_set(bridge, port, "hairpin_mode", hairpin_mode, 0);
+}
+
 static inline void __copy_fdb(struct fdb_entry *ent, 
 			      const struct __fdb_entry *f)
 {
