@@ -48,13 +48,24 @@ static int isbridge(const struct dirent *entry)
 {
 	char path[SYSFS_PATH_MAX];
 	struct stat st;
+	int ret, saved_errno;
 
-	if (entry->d_name[0] == '.')
+	if (entry->d_name[0] == '.'
+	    && (entry->d_name[1] == '\0'
+		|| (entry->d_name[1] == '.'
+		    && entry->d_name[2] == '\0')))
 		return 0;
-	
+
 	snprintf(path, SYSFS_PATH_MAX, 
 		 SYSFS_CLASS_NET "%s/bridge", entry->d_name);
-	return stat(path, &st) == 0 && S_ISDIR(st.st_mode);
+
+	/* Workaround old glibc breakage.
+	   If errno is set, then it fails scandir! */
+	saved_errno = errno;
+	ret = (stat(path, &st) == 0 && S_ISDIR(st.st_mode));
+	errno = saved_errno;
+
+	return ret;
 }
 
 /*
